PROGRAM MargDam;

(*  Climate Framework for Uncertainty, Negotiation and Distribution       *)
(*  Version 1.60  -  27 May 1997                                          *)
(*  Richard S.J. Tol                                                      *)
(*  Institute for Environmental Studies, Vrije Universiteit, Amsterdam    *)

USES
    Crt, RP160, GenRand;

CONST
     consleak = 0.00;                 (* consumption leak *)
     optper = 0;
     runs = 1000;

TYPE
    measure = regarr;
    matmeas = ARRAY [1..NoReg,1..NOI] OF double;
    boolmea = ARRAY [1..NoReg,1..NOI] OF boolean;

VAR
   boolmeas: boolmea;
   regstep, forstep, taxstep: double;
   c2000,c2025,c2050,c2075,c2100: double;
   rhos, rhoa, rhohc, rhoh, rhowt, rhowi, rhoe, rhoi, rhocp, rhodl: double;
   margd: text;

FUNCTION Log(x: double): double;
(* returns ln(x) *)

BEGIN
     IF (x > 1.0E-308) AND (x < 1.0E308) THEN
        log:= ln(x)
     ELSE IF x < 1.0E-308 THEN BEGIN
          log:= ln(1.0E-308);
          IF x < 0 THEN
             faillog:= true
     END
     ELSE
         log:= ln(1.0E38)
END;


FUNCTION Expo(x: double): double;
(* returns exp(x) *)

BEGIN
     IF (x < 709) AND (x > -11356) THEN
        expo:= exp(x)
     ELSE IF x < -11356 THEN
          expo:= exp(-11356)
     ELSE
         expo:= exp(709)
END;

FUNCTION Root(x: double): double;
(* returns sqrt(x) *)

BEGIN
     IF x < 1.0E308 THEN
        root:= sqrt(x)
     ELSE
         root:= 1.0E154
END;

FUNCTION Quad(x: double): double;
(* returns x^2 *)

BEGIN
     IF x < 1.0E154 THEN
        quad:= sqr(x)
     ELSE
         quad:= 1.0E308
END;

FUNCTION Pow(x,y: double): double;
(* returns x^y *)

BEGIN
     pow:= expo(y*log(x))
END;

FUNCTION Interact(M,N: double): double;
(* interaction radiative forcing methane and nitrous oxide (in ppbv) *)

VAR hulp: double;

BEGIN
     hulp:= 1 + pow(M*N,0.75)*2.01E-5 + pow(M*N,1.52)*M*5.31E-15;
     interact:= 0.47*log(hulp)
END;

PROCEDURE InitRho;

BEGIN
     rhos:=  1-1/100 (*exp(hulp/100)*);
     rhoa:=  1-1/10(*exp(hulp/10)*);
     rhohc:= 1-1/15(*exp(hulp/15)*);
     rhoh:=  1-1/5(*exp(hulp/5)*);
     rhoi:=  1-1/5(*exp(hulp/5)*);
     rhoe:=  1-1/5(*exp(hulp/5)*);
     rhowt:= 1-1/10(*exp(hulp/10)*);
     rhowi:= 1-1/50(*exp(hulp/50)*);
     rhodl:= 1-1/50(*exp(hulp/50)*);
     rhocp:= 1-1/50(*exp(hulp/50)*)
END;

PROCEDURE InitRegMult;

VAR i: word;

BEGIN
     economy[1].regmp:= 4.4788/2.5;
     economy[2].regmp:= 4.1401/2.5;
     economy[3].regmp:= 3.0492/2.5;
     economy[4].regmp:= 4.4397/2.5;
     economy[5].regmp:= 4.6320/2.5;
     economy[6].regmp:= 3.5218/2.5;
     economy[7].regmp:= 3.4416/2.5;
     economy[8].regmp:= 3.9997/2.5;
     economy[9].regmp:= 3.9804/2.5;

     FOR i:= 1 TO NoReg DO
         economy[i].regmp:= 1;

     FOR i:= 1 TO NoReg DO
         economy[i].regsq:= economy[i].regmp*economy[i].regmp
END;

PROCEDURE Initialise;

VAR i:word;

BEGIN
     ClrScr;
     faillog:= false;
     InitRho;
     InitRegMult;
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              new(ypcgrowth);
              new(pgrowth);
              new(urbanp);
              new(aeei);
              new(acei);
              new(forestemm)
         END;
     WITH Climate DO BEGIN
          decay[1]:= 1;          (*MERGE*)
          decay[2]:= exp(-1/lifeco2) (*exp(-1/313.8)*);
          decay[3]:= exp(-1/lifeco3) (*exp(-1/79.8)*);
          decay[4]:= exp(-1/lifeco4) (*exp(-1/18.8)*);
          decay[5]:= exp(-1/lifeco5) (*exp(-1/1.7)*);
          frac[1]:= 0.13 (*0.142*);
          frac[2]:= 0.20 (*0.241*);
          frac[3]:= 0.32 (*0.323*);
          frac[4]:= 0.25 (*0.206*);
          frac[5]:= 0.10 (*0.088*);
          new(addco2);
          new(addch4);
          new(addn2o);
          new(scentemp);
          new(ch4em);
          new(n2oem);
          new(acco2);
          new(acch4);
          new(acn2o)
     END;

     Economy[1].name:= 'OECD-A';
     Economy[2].name:= 'OECD-E';
     Economy[3].name:= 'OECD-P';
     Economy[4].name:= 'CEE&SU';
     Economy[5].name:= 'M-E';
     Economy[6].name:= 'L-A';
     Economy[7].name:= 'S&SEA';
     Economy[8].name:= 'CPA';
     Economy[9].name:= 'AFR';

     Economy[1].captrans:= 0;
     Economy[2].captrans:= 0;
     Economy[3].captrans:= 0;
     Economy[4].captrans:= 0;
     Economy[5].captrans:= 0;
     Economy[6].captrans:= 0;
     Economy[7].captrans:= 0;
     Economy[8].captrans:= 0;
     Economy[9].captrans:= 0;

     Economy[1].compens:= 0;
     Economy[2].compens:= 0;
     Economy[3].compens:= 0;
     Economy[4].compens:= 0;
     Economy[5].compens:= 0;
     Economy[6].compens:= 0;
     Economy[7].compens:= 0;
     Economy[8].compens:= 0;
     Economy[9].compens:= 0;

     Economy[1].ypccorr:= 0.421855;
     Economy[2].ypccorr:= 0.263729;
     Economy[3].ypccorr:= 0.244423;
     Economy[4].ypccorr:= 0.306673;
     Economy[5].ypccorr:= 0.373133;
     Economy[6].ypccorr:= 0.437524;
     Economy[7].ypccorr:= 0.196623;
     Economy[8].ypccorr:= 0.121762;
     Economy[9].ypccorr:= 0.724458;

     ReadIncome;
     ReadPopulation;
     ReadYpCGrowth;
     ReadAgrish;
     ReadPGrowth;
     ReadUrbanPop;
     ReadEnergint;
     ReadACEI;
     ReadForestEmm;
     ReadForParam;
     ReadDiscRate;
     ReadTaxPar;
  (*   ReadRegPar; *)
     ReadHurrLife;
     ReadHurrProp;
     ReadFloodDam;
     ReadWindDam;
     ReadHeat;
     ReadCold;
     ReadMalaria;
     ReadWetland;
     ReadDryland;
     ReadProtect;
     ReadSpecies;
     ReadAgricult;
     ReadLeave;
     ReadEnter;
     ReadAddCH4;
     ReadAddN2O;
     ReadCH4Em;
     ReadN2OEm;
     ReadScenGMT;
     WITH climate DO BEGIN
          temps:= modet/6.3/ln(2);
          hurrs:= modeh;
          seas:= modes;
          delaysea:= 1/lifesea;
          delaytemp:= 1/lifetemp;
          floods:= modef;
          winds:= modew;
          ch4decay:= ch4dec;
          n2odecay:= n2odec;
     END;
     bstsq:= quad(baset);
     dbstsq:= quad(dbst);
     bsssq:= quad(bases);
     dbsssq:= quad(dbss);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              ecgrmp:= 1;
              popgrmp:= 1;
              aeeimp:= 1;
              aceimp:= 1;
              foremp:= 1;
              taxmp:= 1;
              formp:= 1;
              (*cregmp:= 1; *)
              specmp:= 1;
              heatmp:= 1;
              coldmp:= 1;
              malarmp:= 1;
              agrimp:= 1;
              hurrlmp:= 1;
              hurrpmp:= 1;
              floodmp:= 1;
              windmp:= 1;
              migmp:= 1;
              protmp:= 1;
              drymp:= 1;
              wetmp:= 1;
         END
END;

PROCEDURE ReInit;

VAR i: word;

BEGIN
     WITH economy[1] DO   (* emf *)
          emissint:= (*0.2567013*) 0.283/0.544565;
     WITH economy[2] DO
          emissint:= (*0.139178*)  0.220/0.46748;
     WITH economy[3] DO
          emissint:= (*0.1144091*) 0.118/0.940698;
     WITH economy[4] DO
          emissint:= (*0.4464261*) 0.413/0.829209;
     WITH economy[5] DO
          emissint:= (*0.4752512*) 0.455/9.147696;
     WITH economy[6] DO
          emissint:= (*0.3097415*) 0.349/0.961393;
     WITH economy[7] DO
          emissint:= (*0.4460469*) 0.492/1.384539;
     WITH economy[8] DO
          emissint:= (*1.6008936*) 1.732/4.530965;
     WITH economy[9] DO
          emissint:= (*0.4555767*) 0.440/1.738977;
     totalpop:= 0;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              inc0:= incbase + captrans;
              income:= inc0;
              population:= pop0;
              welf0:= log(1000*(1-consleak)*inc0/pop0);
              welfare:= welf0;
              YpC:= 1000*inc0/pop0;                     (* ????????? *)
              YpC0:= YpC;
              energint:= 1;                             (* confirmed *)
              energuse:= inc0;
              ryg:= 0;
             (* creg:= 0; *)
              reei:= 0;                                 (* confirmed *)
              rcei:= 0;                                 (* confirmed *)
              emission:= emissint*energuse;
              emiss0:= emission;                        (* confirmed *)
              totalpop:= totalpop + pop0;
              cdr:= 1;
              y:= 1;
              v:= 1;
              p:= 1;
              pg:= popgrmp*pgrowth^[0]/100;
              spec:= 0;
              deadht:= 0;               (*?*)
              deadhr:= 0;
              deadct:= 0;
              deadcr:= 0;
              stress:= 0;
              malaria:= 0;
              malcost:= 0;
              agri1:= 0;
              agri2:= 0;
              hurr1:= 0;
              hurr2:= 0;
              hurr3:= 0;
              hurr4:= 0;
              hurr5:= 0;
              flooddam:= 0;
              winddam:= 0;
              enter:= 0;
              leav:= 0;
              enterc:= 0;
              leavec:= 0;
              coastm:= 0;
              dryld:= 0;
              coastc:= 0;
              drylt:= 0;
              welti:= 0;
              weltt:= 0;
              eloss:= 0;
              sloss:= 0
     END;
     WITH climate DO BEGIN
          temp:= 0.20;
          scentemp^[0]:= temp;
          tsq:= 0;
          dt:= 0;
          dtsq:= 0;
          sea:= 0;
          ssq:= 0;
          ds:= 0;
          dssq:= 0;
          hurr:= 0;
          dh:= 0;
          wind:= 0;
          flood:= 0;
          cbox[1]:= 283.53;
          cbox[2]:= 5.62;
          cbox[3]:= 6.29;
          cbox[4]:= 2.19;
          cbox[5]:= 0.15;
          acco2^[0]:= 0;
          acch4^[0]:= 1222;
          acn2o^[0]:= 296;
          FOR i:= 1 TO 5 DO
              acco2^[0]:= acco2^[0] + cbox[i];
          ch4n2o:= interact(ch4pre,n2opre)
     END
END;

(* TAX PROCEDURES *)

PROCEDURE SetTax2;
(* sets all taxes to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 0 TO CtrMax DO
             economy[i].tax[j]:= 0;
END;

(* REGULATION PROCEDURES *)

(* FORESTRY PROCEDURES *)

PROCEDURE SetInflFor(VAR country: regrec; t: word);
(* determines the influence of COUNTRY.FORESTRY on the economy of COUNTRY *)

VAR fca, fcd, fa, fd, th1, th2: double;

BEGIN
     WITH country DO BEGIN
          IF forestry[optper] > 0 THEN BEGIN
             th1:= forconst - forlinear + root(quad(forconst-forlinear)-4*forconst*forsquare);
             th1:= th1/2/forsquare;
             th2:= (3*forconst/2 - forlinear)/2/forsquare;
             IF forestry[optper] < (th1 + th2) THEN BEGIN
                fd:= forestry[optper];
                fa:= 0
             END
             ELSE IF forestry[optper] < (2*th1 + th2) THEN BEGIN
                  fd:= th1 + th2;
                  fa:= forestry[optper] - fd
             END
             ELSE BEGIN
                  fa:= th1;
                  fd:= forestry[optper] - fa
             END;
             IF fd > foremp*forestemm^[t] THEN BEGIN    (* !!! *)
                fd:= foremp*forestemm^[t];
                fa:= forestry[optper] - fd
             END;
             IF fd > 0 THEN BEGIN
                fcd:= 2*(forconst + forlinear*fd + forsquare*quad(fd))/3;
                IF fcd < forconst*fd THEN
                   fcd:= 2*forconst*fd/3
             END
             ELSE
                 fcd:= 0;
             IF fa > 0 THEN BEGIN
                fca:= forconst + forlinear*fa + forsquare*quad(fa);
                IF fca < forconst*fa THEN
                   fca:= forconst*fa
             END
             ELSE
                 fca:= 0;
             forcost:= fca + fcd
          END
          ELSE
              forcost:= 0
     END
END;

PROCEDURE SetFor2;
(* sets all forestry to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO CtrMax DO
             economy[i].forestry[j]:= 0;
END;

(* SETTINGS *)

PROCEDURE SetMeasures1;
(* sets all measures to zero *)

BEGIN
     SetTax2;
     SetFor2;
END;

(* CALCULATING PROCEDURES *)

PROCEDURE Calculate(VAR country: regrec; t: word);

VAR i, j, decade: word;
    s, h, henter, excessloss, popold, ic, dc, x: double;

BEGIN
     WITH country, climate DO BEGIN
          income:= ((1+pg)*(1+ecgrmp*ypcgrowth^[t-1]/100)-taxmp*ryg)*income;
          y:= income/inc0;
          w1950:= welfare/welf0;
          popold:= population;
          population:= (1+0.01*popgrmp*pgrowth^[t-1])*(population
                       + deadht + deadhr + deadct + deadcr);
          p:= population/pop0;
          YpC:= 1000*income/population;
          v:= ypc/ypc0;
          agrish:= agrish0*pow(v,-0.31);
          energint:= (1 - 0.01*aeeimp*aeei^[t] - reei)*energint;
          energuse:= energint*income;
          emissint:= (1 - 0.01*aceimp*acei^[t] - rcei)*emissint;
          emission:= emissint*energuse;

          IF (t mod 10 = 0) AND (t >= 40) THEN BEGIN
             decade:= trunc((t-40)/10);
             ryg:= taxpar1*tax[decade] + taxpar2*quad(tax[decade]);
             reei:= 0.5*tax[decade];
             rcei:= 0.5*tax[decade]
          END;
          setinflfor(country,t);

          ic:= (1+ypc0/ypccorr/20000)*ypc*ypccorr/ypc0/(1+ypc/20000);
          dc:= (1+ypc0/ypccorr/500)/(1+ypc/500);

(*          edamptc:= edamptc*(1-0.01*(aeeimp*aeei^[t]+reei))*(1-0.01*(aceimp*acei^[t]+rcei));
          sdamptc:= sdamptc*(1-0.01*(aeeimp*aeei^[t]+reei))*(1-0.01*(aceimp*acei^[t]+rcei));
          edamairpo:= edamptc*income*(emission - emiss0)/emiss0;
          sdamairpo:= sdamptc*ic*income*(emission - emiss0)/emiss0; *)

          spec:= specmp*0.5*specl*income*(regmp*abs(dt/dbst)+regsq*sqr(dt/dbst))/60*ic
                 + rhos*spec;
          deadht:= heatmp*heatt*urbanp^[t]*population*regmp*temp/baset;
          deadhr:= heatmp*0.5*heatr*urbanp^[t]*population*(regmp*dt/dbst+regsq*sqr(dt/dbst))/60
                   + rhohc*deadhr;
          deadct:= coldmp*coldt*population*regmp*temp/baset;
          deadcr:= coldmp*0.5*coldr*population*(regmp*dt/dbst+regsq*sqr(dt/dbst))/60
                   + rhohc*deadcr;
          dead:= deadhr+deadht-deadcr-deadct;
          stress:= voslmp*(vhlc + vhll*ypc)*dead/1000;
          malaria:= malarmp*malar*population*regmp*temp/baset*dc;
          dead:= dead + malaria;
          malcost:= voslmp*(vhlc + vhll*ypc)*malaria/1000;
          agri1:= agrimp*dagri*0.5*agrish*income*(regmp*abs(dt/dbst)+regsq*sqr(dt/dbst))/60
                  + rhoa*agri1;
          agri2:= agrimp*agri*agrish*income*regmp*temp/baset;
          x:= 1/6;
          hurr1:= x*hurrlmp*hurrl*population*regmp*hurr/baseh;
          hurr2:= x*hurrpmp*hurrp*income*regmp*hurr/baseh;
          IF dh > 0 THEN BEGIN
             hurr3:= (1-x)*0.5*hurrlmp*hurrl*population*(regmp*dh/dbsh + regsq*sqr(dh/dbsh))/60
                     + rhoh*hurr3;
             hurr4:= (1-x)*0.5*hurrpmp*hurrp*income*(regmp*dh/dbsh + regsq*sqr(dh/dbsh))/60
                     + rhoh*hurr4
          END
          ELSE BEGIN
               hurr3:= 0.2*(1-x)*0.5*hurrlmp*hurrl*population*(regmp*dh/dbsh + regsq*sqr(dh/dbsh))/60
                       + rhoh*hurr3;
               hurr4:= 0.2*(1-x)*0.5*hurrpmp*hurrp*income*(regmp*dh/dbsh + regsq*sqr(dh/dbsh))/60
                       + rhoh*hurr4
          END;
          hurr5:= voslmp*(vhlc + vhll*ypc)*(hurr1+hurr3)/1000;
          dead:= dead + hurr1 + hurr3;
          flooddam:= floodmp*floodd*income*regmp*flood/basef;
          winddam:= windmp*windd*income*regsq*sqr(wind/basew);
          henter:= 0;
          FOR j:= 1 TO NoReg DO
henter:= henter + economy[j].migmp*enter0[j]*economy[j].population*(1+economy[j].ypc0/500)/(1+economy[j].ypc/500);
          enter:= henter*abs(ds/dbss)/60;
          leav:= migmp*leave*population*abs(ds/dbss)/60*dc;
          enterc:= enter*0.40*ypc + rhoi*enterc;
          leavec:= leav*3*ypc + rhoe*leavec;
          x:= 3/4;
          coastm:= x*protmp*coast*income*sea/bases;
          x:= 3/4;
          dryld:= x*drymp*dryl*income*sea/bases/60;
          IF ds > 0 THEN BEGIN
             x:= 3/4;
             coastc:= (1-x)*0.5*protmp*coast*income*(ds/dbss + sqr(ds/dbss))/60
                      +rhocp*coastc;
             x:= 3/4;
             drylt:=  (1-x)*0.5*drymp*dryl*income*(ds/dbss + sqr(ds/dbss))/60
                      +rhodl*drylt;
             x:= 0;
             welti:=  (1-x)*0.5*wetmp*wetl*income*(ds/dbss + sqr(ds/dbss))/60*ic
                      + rhowi*welti;
             weltt:=  x*0.5*wetmp*wetl*income*(ds/dbss + sqr(ds/dbss))/60
                      + rhowt*weltt;
          END
          ELSE BEGIN
               x:= 3/4;
               coastc:= 0.2*(1-x)*0.5*protmp*coast*income*(-ds/dbss + sqr(ds/dbss))/60
                        + rhocp*coastc;
               x:= 3/4;
               drylt:=  0.2*(1-x)*0.5*drymp*dryl*income*(-ds/dbss + sqr(ds/dbss))/60
                        + rhodl*drylt;
               x:= 0;
               welti:=  (1-x)*0.2*0.5*wetmp*wetl*income*(-ds/dbss + sqr(ds/dbss))/60*ic
                        + rhowi*welti;
               weltt:=  x*0.2*0.5*wetmp*wetl*income*(-ds/dbss + sqr(ds/dbss))/60
                        + rhowt*weltt;
          END;
          eloss:= coastm + coastc + dryld + drylt + weltt + enterc
                  + agri1 + agri2 + hurr2 + hurr4 + flooddam + winddam;
          sloss:= spec + stress + malcost + welti + leavec + hurr5;
          income:= income - formp*forcost*y;
          IF i >= 40 THEN
             population:= population - leav + enter - dead;
          pg:= (population-popold)/popold;
          welfare:= (1-consleak)*income - eloss (*- apemp*edamairpo*);
          IF i >= 40 THEN
             income:= income - consleak*(eloss (*+ apemp*edamairpo*));
          IF income < 0 THEN
             income:= 0;
          welfare:= log(1000*welfare/population);
          IF income < 0.00000001 THEN
             welfare:= -100000000
          ELSE
              welfare:= welfare - sloss/income - 0.5*quad(sloss/income)
                        (*- apsmp*sdamairpo/income
                        - 0.5*quad(apsmp*sdamairpo/income)*);
          IF t >= (40+10*optper) THEN
             cdr:= (1+discrate)*cdr;
          IF t = 40+10*optper THEN BEGIN
             NPVY:= income;
             hNPVW:= welfare
          END;
          IF t > (40+10*optper) THEN BEGIN
             NPVY:= NPVY + income/cdr;
             IF hNPVW >= 1.0E-309 THEN
                hNPVW:= hNPVW + welfare/cdr
          END;
          totalpop:= totalpop + population
     END
END;

PROCEDURE Write1(VAR country: regrec; t: word; VAR els,sls,inc,pop: text);

BEGIN
     WITH country, climate DO BEGIN
          IF income > 0.000001 THEN BEGIN
             write(els,eloss/income*100:10:6,' ');
             write(sls,sloss/income*100:10:6,' ')
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ')
          END;
          write(inc,income:12:6,' ');
          write(pop,population:12:6,' ')
     END
END;

PROCEDURE CalcClimate(j: word);
(* calculates the climatic variables at time J *)

VAR mco2: double;
    i: word;
    ht, hdt: double;

BEGIN
     WITH climate DO BEGIN
          mco2:= 0;
          FOR i:= 1 TO NoReg DO
              WITH economy[i] DO BEGIN
                   mco2:= mco2 + emission + foremp*forestemm^[j] - forestry[trunc(j/10)]
              END;
          FOR i:= 1 TO 5 DO
              cbox[i]:= cbox[i]*decay[i] + 0.000471*frac[i]*(mco2 + addco2^[j]);
          acco2^[j]:= 0;
          FOR i:= 1 TO 5 DO
              acco2^[j]:= acco2^[j] + cbox[i];
          acch4^[j]:= acch4^[j-1] + 0.3597*(ch4em^[j] + addch4^[j])
                         - ch4decay*(acch4^[j-1]-ch4pre);
          acn2o^[j]:= acn2o^[j-1] + 0.2079*(n2oem^[j] + addn2o^[j])
                         - n2odecay*(acn2o^[j-1]-n2opre);

          rfCO2:= 6.3*log(acco2^[j]/co2pre);
          rfCH4:= 0.036*(sqrt(acCH4^[j])-sqrt(ch4pre))
                       - interact(acch4^[j],n2opre) + CH4N2O;
          rfN2O:= 0.14*(sqrt(acN2O^[j])-sqrt(N2Opre))
                       - interact(ch4pre,acn2o^[j]) + CH4N2O;

          radforc:= rfCO2 + rfCH4 + rfN2O;
          dt:= delaytemp*temps*radforc - delaytemp*temp;
          temp:= dt + temp;
          flood:= floods*temp;
          wind:= winds*temp;
          tsq:= quad(temp);
          dtsq:= quad(dt);
          ds:= delaysea*seas*temp - delaysea*sea;
          sea:= ds + sea;
          ssq:= quad(sea);
          dssq:= quad(ds);
          hurr:= hurrs*temp;
          dh:= hurrs*dt
     END
END;

PROCEDURE RunEconomy1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,inc,pop:text;

BEGIN
     ReInit;
     assign(els,sessionpathname+'eloss.prn');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.prn');
     rewrite(sls);
     assign(inc,sessionpathname+'income.prn');
     rewrite(inc);
     assign(pop,sessionpathname+'population.prn');
     rewrite(pop);
     FOR i:= 1 TO NYear DO BEGIN
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             Write1(economy[j],i,els,sls,inc,pop)
         END;
         CalcClimate(i);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop)
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop)
END;

PROCEDURE RunEconomy;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,inc,pop:text;

BEGIN
     ReInit;
     assign(els,sessionpathname+'eloss.out');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.out');
     rewrite(sls);
     assign(inc,sessionpathname+'income.out');
     rewrite(inc);
     assign(pop,sessionpathname+'population.out');
     rewrite(pop);
     FOR i:= 1 TO NYear DO BEGIN
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             Write1(economy[j],i,els,sls,inc,pop)
         END;
         CalcClimate(i);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop)
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop)
END;

PROCEDURE Exit;

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              dispose(ypcgrowth);
              dispose(pgrowth);
              dispose(urbanp);
              dispose(aeei);
              dispose(acei);
              dispose(forestemm)
         END;
     WITH Climate DO BEGIN
          dispose(addco2);
          dispose(addch4);
          dispose(addn2o);
          dispose(ch4em);
          dispose(n2oem);
          dispose(acco2);
          dispose(acch4);
          dispose(acn2o)
     END
END;

PROCEDURE ReadAddCO3;
(* reads additional emissions CO2 from addco2.95d *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'addco2.95d');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.addco2^[i]);
     Close(fil)
END;

PROCEDURE CalcMarg;

VAR i, j: word;
    els0, els1, sls0, sls1, inc, pop: text;
    h1, h2, h3, h4, income, popul, inccap: ARRAY [0..NoReg] OF double;
    winc, wpop, winccap, sum0, sum1, sum3, sum5 , sum10: double;
    damdiff: year;

BEGIN
     assign(els0,sessionpathname+'eloss.prn');
     reset(els0);
     assign(els1,sessionpathname+'eloss.out');
     reset(els1);
     assign(sls0,sessionpathname+'sloss.prn');
     reset(sls0);
     assign(sls1,sessionpathname+'sloss.out');
     reset(sls1);
     assign(inc,sessionpathname+'income.prn');
     reset(inc);
     assign(pop,sessionpathname+'population.prn');
     reset(pop);
     FOR i:= 1 TO NYear DO BEGIN
         FOR j:= 0 TO NoReg DO
             read(inc,income[j]);
         readln(inc);
         FOR j:= 0 TO NoReg DO
             read(pop,popul[j]);
         readln(pop);
         FOR j:= 1 TO NoReg DO
             inccap[j]:= income[j]/popul[j]*1000;
         wpop:= 0;
         FOR j:= 1 TO NoReg DO
             wpop:= wpop + popul[j];
         winc:= 0;
         FOR j:= 1 TO NoReg DO
             winc:= winc + income[j];
         winccap:= winc/wpop*1000;
         FOR j:= 0 TO NoReg DO
             read(els0,h1[j]);
         readln(els0);
         FOR j:= 0 TO NoReg DO
             read(els1,h2[j]);
         readln(els1);
         FOR j:= 0 TO NoReg DO
             read(sls0,h3[j]);
         readln(sls0);
         FOR j:= 0 TO NoReg DO
             read(sls1,h4[j]);
         readln(sls1);
         damdiff[i]:= 0;
         FOR j:= 1 TO NoReg DO
             damdiff[i]:= damdiff[i] + (h2[j]+h4[j]-h1[j]-h3[j])*income[j]*winccap/inccap[j];
     END;
     close(els0);
     close(els1);
     close(sls0);
     close(sls1);
     close(inc);
     close(pop);
     sum0:= 0;
     sum1:= 0;
     sum3:= 0;
     sum5:= 0;
     sum10:= 0;
     FOR i:= 40 TO NYear-100 DO BEGIN
         sum0:= sum0 + damdiff[i];
         sum1:= sum1 + damdiff[i]/exp((i-40)*ln(1.01));
         sum3:= sum3 + damdiff[i]/exp((i-40)*ln(1.03));
         sum5:= sum5 + damdiff[i]/exp((i-40)*ln(1.05));
         sum10:= sum10 + damdiff[i]/exp((i-40)*ln(1.10));
     END;
     writeln(margd,sum0:12:8,' ',sum1:12:8,' ',sum3:12:8,' ',sum5:12:8,' ',sum10:12:8)
END;

PROCEDURE DrawRandom;

VAR i, j: word;
    hulp, fake: double;

BEGIN
    WITH climate DO BEGIN
         GenGamma(8.1270,0.3508,temps); (*mode 2.5; var 1*)
(*         GenGamma(4.9218,0.6375,temps); (*mode 2.5; var 2*)
(*         GenGamma(4.27,1.16,temps);       (*mode 3.8; sd 2.4*)
         temps:= temps/6.3/ln(2);
(*         GenGamma(6.3584,0.0317,seas); (* mode 0.17; var 0.08^2 *)
(*         GenGamma(4.0083,0.0565,seas); (* mode 0.17; var 2*0.08^2 *)
(*         GenGamma(7.4729,0.0263,seas);      (* mode 0.17; sd 0.07 *)
         GenGamma(5.9959077,0.0612581,seas); (* mode 0.30604; var 0.15^2 *)
         GenNormal(modeh,0.10,hurrs,fake);
         GenNormal(0,1,floods,winds);
         floods:= modef+modef*floods;
         winds:= modew+modew*winds;
         GenTriang(8,16,1/ch4dec,hulp);
         ch4decay:= 1/hulp;
         GenTriang(100,170,1/n2odec,hulp);
         n2odecay:= 1/hulp;
         GenTriang(0.5*lifetemp,2*lifetemp,lifetemp,hulp);
         delaytemp:= 1/hulp;
         GenTriang(0.5*lifesea,2*lifesea,lifesea,hulp);
         delaysea:= 1/hulp;
         GenNormGTZ(lifeco2,0.25*lifeco2,hulp,fake);
         decay[2]:= exp(-1/hulp);
         GenNormGTZ(lifeco3,0.25*lifeco3,hulp,fake);
         decay[3]:= exp(-1/hulp);
         GenNormGTZ(lifeco4,0.25*lifeco4,hulp,fake);
         decay[4]:= exp(-1/hulp);
         GenNormGTZ(lifeco5,0.25*lifeco5,hulp,fake);
         decay[5]:= exp(-1/hulp);
    END;
    FOR i:= 1 TO NoReg DO
        WITH economy[i] DO BEGIN
             gennormal(1,0.1,ecgrmp,popgrmp);
             gennormal(1,0.1,aeeimp,aceimp);
             gennormal(1,0.1,foremp,fake);

             gennormal(1,0.5,specmp,heatmp);
             gennormal(1,0.5,coldmp,malarmp);
             gennormal(1,0.5,agrimp,hurrlmp);
             gennormal(1,0.5,hurrpmp,floodmp);
             gennormal(1,0.5,windmp,migmp);
             gennormal(1,0.5,protmp,drymp);
             gennormal(1,0.5,wetmp,fake);

             gengamma(2,1,voslmp);

(*             gengamma(2.6180,0.6180,specmp);
             gengamma(2.6180,0.6180,heatmp);
             gengamma(2.6180,0.6180,coldmp);
             gengamma(2.6180,0.6180,malarmp);
             gengamma(2.6180,0.6180,hurrlmp);
             gengamma(2.6180,0.6180,hurrpmp);
             gengamma(2.6180,0.6180,floodmp);
             gengamma(2.6180,0.6180,windmp);
             gengamma(2.6180,0.6180,migmp);
             gengamma(2.6180,0.6180,protmp);
             gengamma(2.6180,0.6180,drymp);
             gengamma(2.6180,0.6180,wetmp);
             gennormal(1,1,agrimp,fake); *)

             gennormGTZ(100,25,rhos,fake);
             rhos:= 1 - 1/rhos;
             gennormGTZ(10,2.5,rhoa,rhowt);
             rhoa:= 1 - 1/rhoa;
             rhowt:= 1 - 1/rhowt;
             gennormGTZ(15,3.75,rhohc,fake);
             rhohc:= 1 - 1/rhohc;
             gennormGTZ(5,1.25,rhoh,rhoi);
             rhoh:= 1 - 1/rhoh;
             rhoi:= 1 - 1/rhoi;
             gennormGTZ(5,1.25,rhoe,fake);
             rhoe:= 1 - 1/rhoe;
             gennormGTZ(50,12.5,rhowi,rhodl);
             rhowi:= 1 - 1/rhowi;
             rhodl:= 1 - 1/rhodl;
             gennormGTZ(50,12.5,rhocp,fake);
             rhocp:= 1 - 1/rhocp;

(*             gengamma(5.8284,0.2071,taxmp);
             gengamma(5.8284,0.2071,formp);
             gengamma(5.8284,0.2071,cregmp);  (* var = 0.25, mode = 1 *)
(*             gengamma(9.8990,0.1124,taxmp);
             gengamma(9.8990,0.1124,formp);
             gengamma(9.8990,0.1124,cregmp);   (* var = 0.125, mode = 1 *)
        END
END;

(* MAIN PROCEDURES *)

PROCEDURE Main;

VAR i: word;

BEGIN
     Initialise;
     RandSeed:= 34567892;
     SetMeasures1;
     assign(margd,sessionpathname+'margdam.prn');
     rewrite(margd);
     ReadAddCO2;
     RunEconomy1;
     ReadAddCO3;
     RunEconomy;
     CalcMarg;
     FOR i:= 1 TO Runs DO BEGIN
         write(i:4,' ');
         DrawRandom;
         ReadAddCO2;
         RunEconomy1;
         ReadAddCO3;
         RunEconomy;
         CalcMarg
     END;
     close(margd);
     Exit
END;

BEGIN
     Main
END.