UNIT RP160Unc;

(* support for FUND1.6Unc programme *)

INTERFACE

USES
    Crt;

CONST
     NYear = 250;                      (* number of years *)
     NoReg = 9;                        (* number of regions *)
     sessionpathname = 'C:\FUND16\DATA\';
     ModeT = 2.5;                      (* modal warming 2xCO2 *)
     BaseT = 3.0;                      (* base case warming *)
     DBsT  = 0.04;                     (* base case yearly warming *)
     ModeS = (*0.17*) 0.30604;         (* modal sea level rise T=1 *)
     BaseS = 0.50;                     (* base case sea level rise T=2.5*)
     DBsS  = 0.0083;                   (* base case yearly rise *)
     ModeH = 0.00;                     (* modal hurricane increase T=1 *)
     BaseH = 0.25;                     (* base case hurricane increase T=2.5 *)
     DBsH  = 0.0042;
     ModeW = 0.024;                    (* modal wind storm T=1 *)
     BaseW = 0.06;                     (* base wind T=2.5 *)
     ModeF = 0.04;                     (* modal river flood T=1 *)
     BaseF = 0.10;                     (* base river T=2.5 *)
     co2pre = 275;
     ch4dec = 1/8.6;                   (* ch4 decay *)
     ch4pre = 790;                     (* ch4 preindustrial *)
     n2odec = 1/120;                   (* n2o decay *)
     n2opre = 285;                     (* n2o preindustrial *)
     c = 0.005;                        (* people enter squared parameter *)
     mme = 0.015;                      (* Middle-East income loss *)
     VHLC = 0 (*250000*);              (* constant value human life *)
     VHLL = 240 (*175*);               (* linear value human life *)
     lifetemp = 50;
     lifesea = 50;
     lifeCO2 = 363;
     lifeCO3 = 74;
     lifeCO4 = 17;
     lifeCO5 = 2;

TYPE
    Year = ARRAY [0..NYear] OF double;
    YrArr = ^Year;
    RegArr = ARRAY [1..NoReg] OF double;
    RegRec = RECORD
                   income : double;       (* income *)
                   incbase : double;      (* base income *)
                   inc0 : double;         (* initial income *)
                   ypccorr: double;       (* correction to 1950 *)
                   captrans : double;     (* capital transfer *)
                   compens : double;      (* compensation *)
                   y : double;            (* income compared to 1950 *)
                   v : double;            (* inc/cap comp to 1950) *)
                   ypcgrowth : yrarr;       (* income growth per year *)
                   ecgrmp : double;       (* multiplier *)
                   ryg : double;          (* reduction inc growth taxing *)
                   taxmp: double;         (* multiplier *)
                   agrish : double;        (* agricultural share *)
                   agrish0: double;
                   welfare : double;      (* welfare *)
                   w1950 : double;        (* welfare compared to 1950 *)
                   welf0 : double;        (* initial welfare *)
                   basewf : double;       (* Pareto minimal welfare *)
                   population : double;   (* population *)
                   pop0 : double;         (* initial population *)
                   p : double;            (* population compared to 1950 *)
                   pg: double;            (* ??? *)
                   YpC : double;          (* income per capita *)
                   YpC0: double;
                   pgrowth : yrarr;       (* population growth per year *)
                   popgrmp : double;      (* multiplier *)
                   urbanp : yrarr;        (* urban population *)
                   aeei : yrarr;          (* autonomous energy efficiency *)
                   aeeimp: double;        (* multiplier *)
                   acei : yrarr;          (* autonomous carbon efficiency *)
                   aceimp: double;        (* multiplier *)
                   reei : double;           (* reduction energy efficiency *)
                   creg : double;            (* costs direct regulation *)
                   regpar1: double;
                   regpar2: double;
                   cregmp: double;           (* multiplier *)
                   rcei : double;           (* reduction carbon efficiency *)
                   emissint : double;       (* GHG emission intensity *)
                   energint : double;       (* energy intensity *)
                   energuse : double;       (* energy use *)
                   emission : double;       (* emissions from fossil fuel *)
                   emiss0 : double;         (* emissions in base year *)
                   forestemm : yrarr;       (* emissions from deforestation, MtCO2 *)
                   foremp: double;          (* multiplier *)
                   forconst : double;       (* forestry cost constant *)
                   forlinear : double;      (* forestry cost linear *)
                   forsquare : double;      (* forestry cost square *)
                   forcost : double;        (* forestry costs *)
                   formp : double;          (* multiplier *)
                   agriprod : double;       (* gross agricultural product *)
(*                   edamairpo : double;      (* tangible loss air pollution *)
(*                   apemp : double;          (* multiplier *)
(*                   edamptc : double;        (* loss per tonne of carbon *)
(*                   edamptc0: double;
(*                   sdamairpo : double;      (* intangible loss air pollution *)
(*                   apsmp : double;          (* multiplier *)
(*                   sdamptc : double;        (* loss per tonne of carbon *)
(*                   sdamptc0: double;*)
                   eloss : double;          (* tangible loss *)
                   sloss : double;          (* intangible loss *)
                   regmp : double;          (* climate multiplier *)
                   regsq : double;          (* clim. mult. squared *)
                   leave : double;          (* number of people leaving *)
                   leav : double;
                   leavec: double;
(*                   leave0 : double;         (* initial number *)
                   enter : double;          (* number of people entering *)
                   enter0 : regarr;         (* initial number + whence *)
                   enterc : double;
                   dead : double;           (* number of deaths *)

                   spec: double;            (* loss species *)
                   deadht: double;          (* heat stress *)
                   deadhr: double;
                   deadct: double;          (* cold stress *)
                   deadcr: double;
                   stress: double;          (* heat + cold stress *)
                   malaria: double;
                   malcost: double;
                   agri1: double;           (* agriculture *)
                   agri2: double;
                   hurr1: double;           (* hurricanes *)
                   hurr2: double;
                   hurr3: double;
                   hurr4: double;
                   hurr5: double;
                   flooddam: double;
                   winddam:  double;
                   coastm: double;          (* maintenance *)
                   coastc: double;          (* construction *)
                   dryld: double;           (* dryland *)
                   drylt: double;
                   welti: double;           (* wetland intangible *)
                   weltt: double;           (* wetland tangible *)
(* parameters *)
                   hurrl: double;           (* life loss natural disasters *)
                   heatt : double;          (* life loss warming *)
                   heatr : double;
                   coldt : double;          (* life gain warming *)
                   coldr : double;
                   malar: double;           (* malaria *)
                   wetl : double;           (* wetland loss *)
                   dryl : double;           (* dryland loss *)
                   coast: double;           (* protection cost *)
                   specl : double;          (* species loss *)
                   hurrp : double;          (* natural disasters *)
                   windd : double;          (* winter storm damage *)
                   floodd: double;          (* flood damage *)
                   agri: double;            (* agricultural loss *)
                   dagri: double;
                   NPVY : double;           (* net present value income *)
                   NPVW : double;           (* net present value welfare *)
                   hNPVW: double;           (* help npv welfare *)
                   discrate : double;       (* discount rate *)
                   cdr : double;            (* composite discount rate *)
                   taxpar1: double;         (* tax cost parameter *)
                   taxpar2: double;
                   name : string;           (* name *)
                   specmp: double;
                   heatmp: double;
                   coldmp: double;
                   malarmp: double;
                   agrimp: double;
                   hurrlmp: double;
                   hurrpmp: double;
                   floodmp: double;
                   windmp: double;
                   migmp: double;
                   protmp: double;
                   drymp: double;
                   wetmp: double;
                   voslmp: double;
             END;
    EconArr = ARRAY [1..NoReg] OF RegRec;

    Box = ARRAY [1..5] OF double;
    ClimRec = RECORD
                    acco2 : yrarr;           (* carbon dioxide *)
                    cbox  : box;             (* co2 boxes *)
                    decay : box;
                    frac  : box;
                    acch4 : yrarr;           (* methane *)
                    acn2o : yrarr;           (* nitrous oxide *)
                    addco2: yrarr;           (* additional co2 emissions *)
                    addch4: yrarr;
                    addn2o: yrarr;
                    scentemp: yrarr;         (* exogenous temperature *)
                    ch4em : yrarr;           (* exogenous methane emissions *)
                    n2oem : yrarr;           (* exogenous n2o emissions *)
                    temps : double;          (* temperature sensitivity *)
                    delaytemp: double;
                    seas  : double;          (* sea level sensitivity *)
                    delaysea: double;
                    hurrs : double;          (* hurricane sensitivity *)
                    floods: double;          (* river flood sensitivity *)
                    winds : double;          (* wind storm sensitivity *)
                    temp  : double;          (* temperature *)
                    tsq   : double;          (* temp squared *)
                    dt    : double;          (* temp difference *)
                    dtsq  : double;          (* dt squared *)
                    sea   : double;          (* sea level *)
                    ssq   : double;          (* sea squared *)
                    ds    : double;          (* sea difference *)
                    dssq  : double;          (* ds squared *)
                    hurr  : double;          (* hurricanes *)
                    dh    : double;          (* hurr diff *)
                    wind  : double;          (* wind storm *)
                    flood : double;          (* river flood *)
                    rfco2 : double;          (* radiative forcing CO2 *)
                    rfch4 : double;          (* rad. forc. CH4 *)
                    rfn2o : double;          (* rad. forc. N2O *)
                    ch4n2o: double;          (* interaction term pre-indus.
                                                rad. forc. CH4-N2O *)
                    radforc: double;
                    ch4decay: double;
                    n2odecay: double;
              END;

VAR
   Economy : EconArr;
   Climate : ClimRec;
   totalpop, totpop0, bstsq, dbstsq, bsssq, dbsssq, rho20, rhocorr: double;

PROCEDURE ReadYpCGrowth;
(* reads economic growth from ypcgrowth.001 *)

PROCEDURE ReadAgrish;
(* reads share of agriculture from agrish.001 *)

PROCEDURE ReadPGrowth;
(* reads population growth from pgrowth.001 *)

PROCEDURE ReadUrbanPop;
(* reads urban population growth from urbanpop.001 *)

PROCEDURE ReadEnergint;
(* reads autonomous energy efficiency improvement from aeei.001 *)

PROCEDURE ReadACEI;
(* reads autonomous carbon efficiency improvements from acei.001 *)

PROCEDURE ReadForestemm;
(* reads emission from deforestation from forestem.001 *)

PROCEDURE ReadForParam;
(* reads reduction cost parameter for forestry from forparam.001 *)

PROCEDURE ReadDiscRate;
(* reads discount rates from discrate.001 *)

PROCEDURE ReadTaxPar;
(* reads tax parameters from taxpar.001 *)

PROCEDURE ReadRegPar;
(* reads energy and carbon regulation parameters from ecepar.001 *)

PROCEDURE ReadHurrLife;
(* reads life-losses hurricanes from hurrlife.001 *)

PROCEDURE ReadHurrProp;
(* reads losses hurricanes from hurrprop.001 *)

PROCEDURE ReadWindDam;
(* reads losses winter storms from winddam.001 *)

PROCEDURE ReadFloodDam;
(* reads losses river floods from flooddam.001 *)

PROCEDURE ReadHeat;
(* reads life-losses global warming from heat.001 *)

PROCEDURE ReadCold;
(* reads life-losses global warming from cold.001 *)

PROCEDURE ReadMalaria;
(* reads life-losses malaria from malaria.001 *)

PROCEDURE ReadWetland;
(* reads wetland losses sea level rise from wetland.001 *)

PROCEDURE ReadDryland;
(* reads dryland losses sea level rise from dryland.001 *)

PROCEDURE ReadProtect;
(* reads protection costs sea level rise from protect.001 *)

PROCEDURE ReadSpecies;
(* reads species loss global warming from species.001 *)

PROCEDURE ReadAgricult;
(* reads agricultural losses from agricult.001 *)

(*PROCEDURE ReadEDamptC;
(* reads tangible losses per tonne carbon air pollution from edam.001 *)

(*PROCEDURE ReadSDamptC;
(* reads intangible losses per tonne carbon air pollution from sdam.001 *)

PROCEDURE ReadIncome;
(* reads base income from inc0.001 *)

PROCEDURE ReadPopulation;
(* reads initial population from pop0.001 *)

PROCEDURE ReadLeave;
(* reads numbers leaving from leave.001 *)

PROCEDURE ReadEnter;
(* reads numbers entering from enter.001 *)

PROCEDURE ReadAddCO2;
(* reads additional emissions CO2 from addco2.001 *)

PROCEDURE ReadAddCH4;
(* reads additional emissions CH4 from addch4.001 *)

PROCEDURE ReadAddN2O;
(* reads additional emissions N2O from addn2o.001 *)

PROCEDURE ReadCH4Em;
(* reads additional CH4 emissions from ch4em.001 *)

PROCEDURE ReadN2OEm;
(* reads additional N2O emissions from n2oem.001 *)

PROCEDURE ReadScenGMT;
(* reads exogenous temperatures from scengmt.001 *)

IMPLEMENTATION

PROCEDURE ReadYpCGrowth;
(* reads economic growth from ypcgrowth.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'ypcgrowth.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].ypcgrowth^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadAgrish;
(* reads share of agriculture from agrish.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'agrish.001');
     Reset(fil);
     FOR j:= 1 TO NoReg DO
         ReadLn(fil,Economy[j].agrish0);
     Close(fil)
END;

PROCEDURE ReadPGrowth;
(* reads population growth from pgrowth.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'pgrowth.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].pgrowth^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadUrbanPop;
(* reads urban population from urbanpop.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'urbanpop.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].urbanp^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadEnergint;
(* reads autonomous energy efficiency improvement from aeei.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'aeei.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].aeei^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadACEI;
(* reads autonomous carbon efficiency improvements from acei.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'acei.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].acei^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadForestemm;
(* reads emission from deforestation from forestemm.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'forestemm.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,Economy[j].forestemm^[i]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadForParam;
(* reads reduction cost parameter for forestry from forparam.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'forparam.001');
     Reset(fil);
     FOR j:= 1 TO NoReg DO BEGIN
         WITH Economy[j] DO
              Read(fil,forconst,forlinear,forsquare);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadDiscRate;
(* reads discount rates from discrate.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'discrate.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].discrate);
     Close(fil)
END;

PROCEDURE ReadTaxPar;
(* reads tax parameters from taxpar.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'taxpar.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO BEGIN
         Read(fil,economy[i].taxpar1);
         ReadLn(fil,economy[i].taxpar2)
     END;
     Close(fil)
END;

PROCEDURE ReadRegPar;
(* reads energy and carbon regulation parameters from ecepar.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'ecepar.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO BEGIN
         Read(fil,economy[i].regpar1);
         ReadLn(fil,economy[i].regpar2)
     END;
     Close(fil)
END;

PROCEDURE ReadHurrlife;
(* reads life-losses hurricanes from hurrlife.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'hurrlife.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].hurrl);
     Close(fil)
END;

PROCEDURE ReadHurrProp;
(* reads losses hurricanes from hurrprop.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'hurrprop.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].hurrp);
     Close(fil)
END;

PROCEDURE ReadWindDam;
(* reads losses winter storms from winddam.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'winddam.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].windd);
     Close(fil)
END;

PROCEDURE ReadFloodDam;
(* reads losses river floods from flooddam.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'flooddam.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].floodd);
     Close(fil)
END;

PROCEDURE ReadHeat;
(* reads life-losses global warming from heat.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'heat.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              Read(fil, heatt);
              Readln(fil, heatr)
         END;
     Close(fil)
END;

PROCEDURE ReadCold;
(* reads life-losses global warming from cold.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'cold.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              Read(fil, coldt);
              Readln(fil, coldr)
         END;
     Close(fil)
END;

PROCEDURE ReadMalaria;
(* reads life-losses global warming from malaria.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'malaria.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              Readln(fil, malar);
     Close(fil)
END;

PROCEDURE ReadWetland;
(* reads wetland losses sea level rise from wetland.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'wetland.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,wetl);
     Close(fil)
END;

PROCEDURE ReadDryland;
(* reads dryland losses sea level rise from dryland.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'dryland.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].dryl);
     Close(fil)
END;

PROCEDURE ReadProtect;
(* reads protection costs sea level rise from protect.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'protect.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,coast);
     Close(fil)
END;

PROCEDURE ReadSpecies;
(* reads species loss global warming from species.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'species.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              ReadLn(fil,specl);
     Close(fil)
END;

PROCEDURE ReadAgricult;
(* reads agricultural losses global warming from agricult.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'agricult.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO BEGIN
         Read(fil,economy[i].agri);
         ReadLn(fil,economy[i].dagri)
     END;
     Close(fil)
END;

(*PROCEDURE ReadEDamptC;
(* reads tangible losses per tonne carbon air pollution from edam.001 *)
(*
VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'EDam.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].EDamptC0);
     Close(fil)
END;

PROCEDURE ReadSDamptC;
(* reads intangible losses per tonne carbon air pollution from sdam.001 *)
(*
VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'SDam.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].SDamptC0);
     Close(fil)
END;*)

PROCEDURE ReadIncome;
(* reads base income from inc0.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'inc0.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].incbase);
     Close(fil)
END;

PROCEDURE ReadPopulation;
(* reads initial population from pop0.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'pop0.001');
     Reset(fil);
     TotPop0:= 0;
     FOR i:= 1 TO NoReg DO BEGIN
         ReadLn(fil,economy[i].pop0);
         totpop0:= totpop0 + economy[i].pop0
     END;
     Close(fil)
END;

PROCEDURE ReadLeave;
(* reads numbers leaving from leave.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'leave.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO
         ReadLn(fil,economy[i].leave);
     Close(fil)
END;

PROCEDURE ReadEnter;
(* reads numbers entering from enter.001 *)

VAR fil: text;
    i,j: word;

BEGIN
     Assign(fil,sessionpathname+'enter.001');
     Reset(fil);
     FOR i:= 1 TO NoReg DO BEGIN
         FOR j:= 1 TO NoReg DO
             Read(fil,economy[i].enter0[j]);
         ReadLn(fil)
     END;
     Close(fil)
END;

PROCEDURE ReadAddCO2;
(* reads additional emissions CO2 from addco2.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'addco2.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.addco2^[i]);
     Close(fil)
END;

PROCEDURE ReadAddCH4;
(* reads additional emissions CH4 from addch4.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'addch4.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.addch4^[i]);
     Close(fil)
END;

PROCEDURE ReadAddN2O;
(* reads additional emissions N2O from addn2o.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'addn2o.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.addn2o^[i]);
     Close(fil);
     FOR i:= 0 TO NYear DO
         climate.addn2o^[i]:= climate.addn2o^[i]*14/22

END;

PROCEDURE ReadCH4Em;
(* reads exogenous emissions CH4 from ch4em.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'ch4em.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.ch4em^[i]);
     Close(fil);
(*     FOR i:= 41 TO NYear DO
         climate.ch4em^[i]:= climate.ch4em^[i] + 69; *)
END;

PROCEDURE ReadN2OEm;
(* reads exogenous N2O emissions from n2oem.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'n2oem.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.n2oem^[i]);
     Close(fil);
     FOR i:= 41 TO NYear DO
         climate.n2oem^[i]:= (climate.n2oem^[i])*14/22
END;

PROCEDURE ReadScenGMT;
(* reads exogenous temperatures from scengmt.001 *)

VAR fil: text;
    i: word;

BEGIN
     Assign(fil,sessionpathname+'scengmt.001');
     Reset(fil);
     FOR i:= 0 TO NYear DO
         ReadLn(fil,climate.scentemp^[i]);
     Close(fil)
END;

END.
